// #################################
//
//           Prescribble
//
//       (C)opyright 2001-2003
//           by Kent Dahl 
//
//        MenThal@bigfoot.com
//  http://www.bigfoot.com/~MenThal/
//
//    Released under the GPL 
//     (Gnu Public License)
//
//       Use at own risk. 
//
// ##################################

#include <cstring>
#include <cstdlib>
#include <ctime>
#include <cstdio>
#include <vector>

// I18N
#include <flpda/gettext.h>

#include <flpda/Widget_Factory.h>

#include <FL/Fl.H>
#include <FL/Fl_Widget.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Browser.H>
#include <FL/Fl_Scroll.H>
#include <FL/Fl_Pack.H>
#include <FL/Fl_Counter.H>

#include <FL/Fl_Menu_Button.H>
#include <FL/Fl_Light_Button.H>
#include <FL/Fl_Value_Slider.H>


#include <FL/fl_draw.H>
#include <FL/fl_message.H>

//#include <expat.h>

#include <algorithm>

//#include <flpda/Widget_Factory.h>

#define my_fl_file_chooser fl_ag_file_chooser
#include "modifiedFLTK/fl_ag_file_chooser.H"

#ifdef FLPDA
// #include "My_About_Dialog.h"
#endif

#include "my_iostream.hpp"

#define VERSION "0.14"

#include "prescribble.hpp"
#include "prescrib_fltk.hpp"

#include "prescribblereader.hpp"

#define XSIZE 160
#define YSIZE 240
#define MENUYSIZE 16

#ifndef DEBUG
#define DEBUG 0
#endif


namespace Prescribble {


//============ Application {
void Application::resetDefaultNotes()
{
    for(int i = 0;i<4;i++)
        m_notes.push_back( new Note(m_config, this ) );
    m_currentNote = 0;
}
 
void Application::removeNotes(){
    while( !m_notes.empty())
    {
        delete m_notes.back();
        m_notes.pop_back();
    }
}

    Application::Application( Config &a_conf)
        : m_pFeedbackString(0),
          m_config(a_conf),
          m_savingQuitter(     this, &Application::saveAndQuit      ),
          m_nonsavingQuitter(  this, &Application::quitWithoutSaving ),
          m_loader(      this, &Application::loadNotes      ),
          m_loaderFrom(  this, &Application::loadNotesFrom  ),
          m_makeNew(     this, &Application::clearNotes     ),
          m_saver(       this, &Application::saveNotes      ),
          m_saverAs(     this, &Application::saveNotesTo    ),
          m_about(       this, &Application::about          ),
          m_toPrev(      this, &Application::prevNote       ),
          m_toNext(      this, &Application::nextNote       ),
          m_inserter(    this, &Application::insertNote, this ),
          m_appender(    this, &Application::insertNote,0   ),

          m_noteForward( this, &Application::moveNote, this ),
          m_noteBackward( this, &Application::moveNote, 0   ),
          m_noteDelete(  this, &Application::deleteNote     ),

          m_configOpener(  this, &Application::openConfig   )

         // m_configController( m_config )



        {
            resetDefaultNotes();
        }


void Application::start()
{
    if(m_config.quietLoadOnEntry)
        loadNotes(0);
    m_pArea->redrawArea(0);
}

void Application::writeto( my_ostream & out ){
    NotesVector::iterator it    = m_notes.begin();
    NotesVector::iterator itend = m_notes.end();
    
    std::fprintf(out, "<notebook>\n");
    while( it != itend )
    {
	(*it)->writeto( out );
	it++;
    }
    std::fprintf(out, "</notebook>\n");
}



int Application::handleKeyboard(int evt)
{
    return 0;
}



int Application::saveNotesTo(void*)
{
    char *file = my_fl_file_chooser(_("Save notes to..."), 
                                 "*.dat*", m_config.filename.c_str() );

    int result = 0;
    if(file!=0)
    {
        // if(DEBUG)std::cout<<"file: "<<file<<"\n";
        m_config.filename = file;
        saveNotes(0);
        result = 1;
    }
    m_pArea->redrawArea(0);
    return result;
}
int Application::saveNotes(void*)
{

    setFeedbackLabel(_("saving"));
    const char *filename = m_config.filename.c_str();

    // zip hack
    bool doZipHack = m_config.allowZipUsage 
        && std::strstr(filename,m_config.zipExtension)!=0;
    if(doZipHack){
        filename = m_config.unzipTmpFile;
    }
    std::printf(_("filename: %s\n"), filename);

    // scope to make file close...
    {
	FILE *out = fopen( filename, "w" );
	if(out==NULL)
	{
	    std::printf(_("can't write: %s\n"), filename);
	    return 0;
	}
	writeto( out );
	fclose( out );
    }

    if(doZipHack){
        //gzip it
        std::string s("gzip -f ");
        s+=filename;
        if(DEBUG)std::printf("system: %s\n",s.c_str());
        std::system( s.c_str() );

        // copy to real location
        s = "cp ";
        s+= m_config.unzipTmpFile; 
        s+= m_config.zipExtension; 
        s+= " ";
        s+= m_config.filename.c_str();  

        if(DEBUG)std::printf("system: %s\n",s.c_str());
        std::system( s.c_str() );

        // remove temp file
        s  = "rm ";
        s += m_config.unzipTmpFile; s += m_config.zipExtension ;
        if(DEBUG)std::printf("system: %s\n",s.c_str());
        std::system( s.c_str() );

    } 


    feedbackNotePage();
    m_pArea->setDirty(true);


    return 0;
}


int Application::loadNotesFrom(void*)
{
    fl_message_font(0,8);
    char *file = my_fl_file_chooser(_("Load notes from..."), 
                                 "*.dat", m_config.filename.c_str() );

    if(file!=0)
    {
        // if(DEBUG)std::cout<<"file: "<<file<<"\n";
        m_config.filename = file;
        loadNotes(0);
    }
    m_pArea->redrawArea(0);
    m_pArea->setDirty(true);
    return 0;

}






int Application::loadNotes(void*)
{
    setFeedbackLabel(_("loading"));
    // clear out the old first?
    removeNotes();

    const char* filename = m_config.filename.c_str();

    bool doZipHack = m_config.allowZipUsage 
        && std::strstr(filename,m_config.zipExtension)!=0;
    if(doZipHack){

        std::string s = "cp ";
        s+= m_config.filename.c_str();  
        s+= " ";
        s+= m_config.unzipTmpFile; 
        s+= m_config.zipExtension; 
        if(DEBUG)std::printf("system: %s\n",s.c_str());
        std::system( s.c_str() );

        //gzip it
        s = "gunzip -f ";
        s+= m_config.unzipTmpFile; 
        s+= m_config.zipExtension; 
        if(DEBUG)std::printf("system: %s\n",s.c_str());
        std::system( s.c_str() );


        filename = m_config.unzipTmpFile;
    }

    // actual writing
    XMLReader reader(m_config, this);
    bool readOk =
        reader.loadInto( filename, m_notes );
    
    if(doZipHack){
        //cleanup
        // remove temp file
        std::string s  = "rm ";
        s += m_config.unzipTmpFile; //s += m_config.zipExtension ;
        if(DEBUG)std::printf("system: %s\n",s.c_str());
        std::system( s.c_str() );
    }

    
    if(DEBUG)printf("reading..done!!!!!!\n");
    

    m_currentNote=0;
    
    //what if something failed terribly?
    if( m_notes.size()<1 || !readOk )
    {
        resetDefaultNotes();
        m_pArea->setCurrentNote( m_notes[m_currentNote] );
        
        if(DEBUG) std::printf("No notes loaded! Notes cleared.");
        fl_message_font(0,8);
        fl_alert(_("Load error.\nNotes cleared."));
        //setFeedbackLabel("load error");               
    }
    feedbackNotePage();
    //if(DEBUG)printf("feedbackNotePage...\n");
    m_pArea->setCurrentNote( m_notes[m_currentNote] );
    m_pArea->setDirty(true);
    return 0;
}



int Application::clearNotes(void*)
{
    if(fl_ask(_("Do you really want to\n"
              "delete ALL the notes?")))
    {
        removeNotes();
        resetDefaultNotes();
        m_pArea->setCurrentNote( m_notes[m_currentNote] );
        m_pArea->redrawArea(0);
        feedbackNotePage();
        m_pMainWindow->redraw();
    }
    return 1;
}




    int Application::saveAndQuit(void*){
        if(m_config.quietSaveOnExit){
            saveNotes(NULL);
        } else {
            if( saveNotesTo(NULL) ){
                // saved OK, lets quit
            } else {
                // save cancel, don't quit
                return 0;
            }                       
        }
        
        delete m_pMainWindow;
        return 0;
    }



int Application::quitWithoutSaving(void*){
    if( !m_config.quietSaveOnExit || fl_ask(_("Do you really want to quit?")))
    {
        delete m_pMainWindow;
    }
    return 0;
}





void  Application::done_cb(Fl_Widget* w, void* data) {
	Fl_Window* window = reinterpret_cast<Fl_Window*>(data);
	// window->hide();
	delete window;
}
Fl_Window* Application::createAboutDialog( 
    const char* application, const char* version )
{
    Fl_App_Window* window = Widget_Factory::new_window(_("About"));
	window->set_modal();
        
	Fl_Browser* b = Widget_Factory::new_select_browser(
            0, 0,
            Fl_Group::current()->w(), Fl_Group::current()->h());
	Fl_Group::current()->resizable(b);
	b->callback(done_cb, window);
	b->add("");
	char msg[256];
	sprintf(msg, "@c@b%s %s", application, version);
	b->add(msg);
	b->add("");

	b->add("@cCopyright (C) 2001-2003");
	b->add("@cKent Dahl");
	b->add("@c<kentda@pvv.org>");

	b->add("");
	b->add("@cLicensed under the");
	b->add("@cGNU General Public License.");
	b->add("@cThis is free software, and you");
	b->add("@care welcome to redistribute it");
	b->add("@cunder certain conditions.");

	b->add("");
	sprintf(msg, "@c%s comes with", application);
	b->add(msg);
	b->add("@cABSOLUTELY NO WARRANTY.");

	window->end();

	Fl_Dockable_Window* toolbar = Widget_Factory::new_toolbar();
	Widget_Factory::new_button(_("Done"), done_cb, window);
	toolbar->end();

	window->add_dockable(toolbar, 1);

	window->show();
        return(window);
}


    int Application::about(void*){

#ifdef FLPDA
        createAboutDialog("Prescribble", VERSION );
#else
        fl_message_font(0,8);
        fl_message("--<[ Prescribble ]>--\n"
                   "  (C)2001 Kent Dahl");
#endif
        return 0;
    }


void Application::setFeedbackLabel( const char *a_str )
{
    delete m_pFeedbackString;
    m_pFeedbackString = strdup(a_str);
    m_pFeedback->label( m_pFeedbackString );
    m_pFeedback->redraw();
    //m_pMainWindow->redraw(); //
}


    void Application::feedbackNotePage(){
        delete m_pFeedbackString;
        m_pFeedbackString = strdup("%i/%i\0\0\0\0\0");
        std::sprintf(m_pFeedbackString,
                     "%i/%i", m_currentNote+1,m_notes.size() );
        m_pFeedback->label( m_pFeedbackString );
        m_pFeedback->redraw();


        if(m_currentNote == 0)
            m_pLeftButton->deactivate();
        else
            m_pLeftButton->activate();

        if( static_cast<unsigned int>(m_currentNote) == m_notes.size()-1 )
            m_pRightButton->deactivate();
        else
            m_pRightButton->activate();

    }

    int Application::nextNote(void*){
        if (m_currentNote < (int)m_notes.size() - 1)
            m_currentNote++;
        m_pArea->setCurrentNote( m_notes[m_currentNote] );
        feedbackNotePage();
        return 0;
    }

    int Application::prevNote(void*){
        if (m_currentNote > 0)
            m_currentNote--;
        m_pArea->setCurrentNote( m_notes[m_currentNote] );
        feedbackNotePage();
        return 0;
    }

    int Application::insertNote(void*a){
        if(!a){
            m_notes.push_back( new Note(m_config, this) );
            m_currentNote = m_notes.size()-1;
        }
        else
        {
            m_notes.insert( find(m_notes.begin(), m_notes.end(),
                                 m_notes[m_currentNote]),
                            new Note(m_config, this));
        }
        m_pArea->setCurrentNote( m_notes[m_currentNote] );
        feedbackNotePage();
        return 0;
    }

int Application::deleteNote(void*)
{
    if(fl_ask(_("Do you really want to\n"
              "delete the current note?")))
    {
        if(m_notes.size()>1){
            
            m_notes.erase( find(m_notes.begin(), m_notes.end(),
                                m_notes[m_currentNote]) );
            if(m_notes.size()<=static_cast<unsigned int>(m_currentNote)) m_currentNote--;
        }else{
            m_pArea->cleanArea(0);
        }

    }
    m_pArea->setCurrentNote( m_notes[m_currentNote] );
    feedbackNotePage();
    return 0;

    return 0;
}
int Application::moveNote(void* a)
{

    if(a){
        if( static_cast<unsigned int>(m_currentNote+1)<m_notes.size() ){
            Note *note = m_notes[m_currentNote];
            m_notes[m_currentNote] = m_notes[m_currentNote+1];
            m_notes[m_currentNote+1] = note;
            m_currentNote++;
        }
        else
        {
            m_appender.callback();
            m_currentNote--;
        }
    }
    else
    {
        if( m_currentNote>0 ){
            Note *note = m_notes[m_currentNote];
            m_notes[m_currentNote] = m_notes[m_currentNote-1];
            m_notes[m_currentNote-1] = note;
            m_currentNote--;
        }
        else
        {
            m_inserter.callback();
            m_currentNote++;
        }
    }
    m_pArea->setCurrentNote( m_notes[m_currentNote] );
    feedbackNotePage();
    return 0;

}




    //int Application::setConfig(void*){  return 0; }
int Application::openConfig(void*){
    ConfigController *cc = 
        new ConfigController( m_config );
    cc->openWindow();
    return 0; 
}





void Application::zoom( int way )
{
    m_pArea->zoom(way);

}
void Application::toolSelect( int way )
{
    m_pEditMenuButton->picked( m_pToolboxMenu->popup(0,0) );

}



Fl_Menu_Item* Application::getFilePopupMenu()
{   
    static Fl_Menu_Item filePopupMenu[] =
            {
                { _("Save and quit"),0, &CallbackBase::cb, &m_savingQuitter},
                { _("Quit"),0, &CallbackBase::cb, &m_nonsavingQuitter,
                  FL_MENU_DIVIDER},
                { _("Save As..."),0, &CallbackBase::cb, &m_saverAs },
                { _("Save"),0, &CallbackBase::cb, &m_saver },
                { _("New"), 0, &CallbackBase::cb, &m_makeNew },
                { _("Load"),0, &CallbackBase::cb, &m_loader, },
                { _("Load from..."),0, &CallbackBase::cb, &m_loaderFrom,
                  FL_MENU_DIVIDER },
                { _("About"),0, &CallbackBase::cb, &m_about },


//                { "...", 0, 0,0, FL_SUBMENU },
//                { "Save",0, &CallbackBase::cb, &m_saver },
//                { "Load",0, &CallbackBase::cb, &m_loader },
//                {0},

                {0}
            };
    return filePopupMenu;

}


Fl_Menu_Item* Application::getEditPopupMenu(PrescribbleArea *box)
{ 

static Fl_Menu_Item editPopupMenu[] =
{
    {_("Clear"),0,&CallbackBase::cb,&box->m_cleaner},
 //   {"Redraw",0,&CallbackBase::cb,&box->m_redrawer},

    {_("Undo"),0,&CallbackBase::cb,&box->m_undoer},
    {_("Redo"),0,&CallbackBase::cb,&box->m_redoer,
     FL_MENU_DIVIDER},
    
    {_("Note..."),0,0,0,
     FL_SUBMENU},
    {_("Delete"),0,&CallbackBase::cb,&m_noteDelete },
    {_("Insert"),0,&CallbackBase::cb,&m_inserter },
    {_("Append"),0,&CallbackBase::cb,&m_appender,
     FL_MENU_DIVIDER},
    
    {_("Backward"),0,&CallbackBase::cb,&m_noteBackward },
    {_("Forward"),0,&CallbackBase::cb,&m_noteForward },


    {0},
    {_("Config"),0,&CallbackBase::cb,&m_configOpener},
    {_("Redraw"),0,&CallbackBase::cb,&box->m_redrawer},
    {_("Mode..."),0,0,0,
     FL_SUBMENU},
    

    { _("Delete"), 0,&CallbackBase::cb,&box->m_eraserDel},
    { _("Erase Box"),  0,&CallbackBase::cb,&box->m_eraser},//FL_MENU_INACTIVE},
    { _("Erase Drag"), 0,&CallbackBase::cb,&box->m_dragEraser},
    { _("Zoom"), 0,&CallbackBase::cb,&box->m_setZoomMode},
    { _("Ellipse"), 0,&CallbackBase::cb,&box->m_setEllipseMode},
    { _("Box"),   0,&CallbackBase::cb,&box->m_setBoxMode},
    { _("Draw"),   0,&CallbackBase::cb,&box->m_setDrawMode},
    
//                    { "Erase",0,&CallbackBase::cb,&box->m_eraser},
    
//                     FL_MENU_DIVIDER},

    {0},
//    {"Redraw",0,&CallbackBase::cb,&box->m_redrawer},
    {0}
};
 m_pToolboxMenu = &editPopupMenu[13];

 return editPopupMenu;
}



#ifdef FLPDA
 // specific code that uses Flek and flpda
#include <flpda/Widget_Factory.h>


Fl_Window *Application::createArea( )
{
  //std::printf("Application::createArea debug # 0\n");
    Fl_App_Window *window; 

    Fl::set_color(FL_SELECTION_COLOR, 0, 0, 0); 
    
    window = new ApplicationWindow( XSIZE, 
                                 YSIZE, 
                                 "Prescribble", this);
     window->color(FL_WHITE);
     window->resizable(window);       


    m_pMainWindow = window;
    
    //std::printf("Application::createArea debug # 1\n");

    PrescribbleArea *box 
        = new PrescribbleArea( m_notes[m_currentNote], 
                               0,0, 
                               XSIZE , YSIZE ,
                               m_config ,this);
    m_pArea = box;
    window->end();
    
    Fl_Dockable_Window* toolbar = WidgetFactory::new_toolbar();
    
    Fl_Menu_Button *mbut
        = Widget_Factory::new_menu_button( 
            0,YSIZE-MENUYSIZE,
            _("File")
            );
    
    mbut->menu(getFilePopupMenu());
    
      
    mbut
        = Widget_Factory::new_menu_button( 
            XSIZE/6 ,YSIZE-MENUYSIZE, // XSIZE/6,MENUYSIZE,
            _("Edit")
            );

    mbut->menu( getEditPopupMenu(box) );
    m_pEditMenuButton = mbut;

    new Fl_Box( FL_NO_BOX, 0,0,6,6,"");//spacing
    
    // UNDO
    Fl_Button *but
        = Widget_Factory::new_button( 
            2*XSIZE/6 + XSIZE/(6*2*2)  ,YSIZE-MENUYSIZE, 
            "@|<"
            );
    but->labelsize(8);
    but->size( XSIZE/(6*2), Widget_Factory::buttonheight() );
    but->labeltype(FL_SYMBOL_LABEL);
    but->callback( &CallbackBase::cb,&box->m_undoer );
    m_pUndoButton = but;
    
    // REDO
    but
        = Widget_Factory::new_button( 
            2*XSIZE/6 + XSIZE/(6*2*2/3) ,YSIZE-MENUYSIZE, 
            "@>|"
            );
    but->labelsize(8);
    but->size( XSIZE/(6*2), Widget_Factory::buttonheight() );
    but->labeltype(FL_SYMBOL_LABEL);
    but->callback( &CallbackBase::cb,&box->m_redoer );
    m_pRedoButton = but;
    
    
    new Fl_Box( FL_NO_BOX, 0,0,6,6,""); //spacing
    
    
    // PREV
    but 
        = Widget_Factory::new_button( 
            4*XSIZE/6 ,YSIZE-MENUYSIZE, 
            "@<-");
    but->size( XSIZE/(6*2), Widget_Factory::buttonheight() );
    but->callback( &ApplicationCallback::cb, &m_toPrev );
    but->labeltype(FL_SYMBOL_LABEL);
    
    m_pLeftButton = but;
    
    //FEEDBACK
    m_pFeedback
        = new Fl_Button( //FL_DOWN_BOX,
            4*XSIZE/6+XSIZE/(6*2) ,YSIZE-MENUYSIZE, 
            XSIZE-2*XSIZE/(6*2)-4*XSIZE/6,MENUYSIZE,
            "...");

    m_pFeedback->labelsize(8);
    m_pFeedback->size( XSIZE-2*XSIZE/(6*2)-4*XSIZE/6, 
                       Widget_Factory::buttonheight() );
    m_pFeedback->callback(&CallbackBase::cb,&box->m_redrawer);
    
    // NEXT
    but 
        = Widget_Factory::new_button( 
            XSIZE-XSIZE/(6*2) ,YSIZE-MENUYSIZE, 
            "@->");
    but->labeltype(FL_SYMBOL_LABEL);
    but->size( XSIZE/(6*2), Widget_Factory::buttonheight() );
    but->callback( &ApplicationCallback::cb, &m_toNext );
    m_pRightButton = but;
    
    toolbar->end();
    window->add_dockable(toolbar, 1);
// ===============================================
// END FLPDA version
#else
// ===============================================
// START plain FLTK verison
    
     Fl_Window *Application::createArea( )
        {
            Fl_Window *window 
                = new ApplicationWindow( XSIZE, YSIZE,"Prescribble", this);

            m_pMainWindow = window;

            PrescribbleArea *box 
                = new PrescribbleArea( m_notes[m_currentNote], 
                                       0,0, 
                                       XSIZE, YSIZE-MENUYSIZE,
                                       m_config , this);
            
            m_pArea = box;

            Fl_Menu_Button *mbut
                 = new Fl_Menu_Button(              
                     0,YSIZE-MENUYSIZE, XSIZE/6,MENUYSIZE,
                     _("File")
                     );

            mbut->labelsize(8);
            mbut->menu(getFilePopupMenu());
            mbut
                = new Fl_Menu_Button(
                    XSIZE/6 ,YSIZE-MENUYSIZE,  XSIZE/6,MENUYSIZE,
                    _("Edit")
                    );
            mbut->labelsize(8);

            mbut->menu( getEditPopupMenu(box) );
            m_pEditMenuButton = mbut;


            // UNDO
            Fl_Button *but
                = new Fl_Button(
                    2*XSIZE/6 + XSIZE/(6*2*2)  ,YSIZE-MENUYSIZE, 
                    XSIZE/(6*2),MENUYSIZE,
                    "@|<"
                    );
            but->labelsize(8);
            but->labeltype(FL_SYMBOL_LABEL);
            but->callback( &CallbackBase::cb,&box->m_undoer );
            m_pUndoButton = but;

            // REDO
           but
                = new Fl_Button(
                    2*XSIZE/6 + XSIZE/(6*2*2/3) ,YSIZE-MENUYSIZE, 
                    XSIZE/(6*2),MENUYSIZE,
                    "@>|"
                    );
            but->labelsize(8);
            but->labeltype(FL_SYMBOL_LABEL);
            but->callback( &CallbackBase::cb,&box->m_redoer );
            m_pRedoButton = but;


            new Fl_Box( FL_NO_BOX, 0,0,6,6,""); //spacing


            // PREV
            but 
                = new Fl_Button(
                    4*XSIZE/6 ,YSIZE-MENUYSIZE, 
                    XSIZE/(6*2),MENUYSIZE,
                    "@<-");
            but->callback( &ApplicationCallback::cb, &m_toPrev );
            but->labeltype(FL_SYMBOL_LABEL);
            m_pLeftButton = but;


            //FEEDBACK
            m_pFeedback
                = new Fl_Button( //FL_DOWN_BOX,
                             4*XSIZE/6+XSIZE/(6*2) ,YSIZE-MENUYSIZE, 
                             XSIZE-2*XSIZE/(6*2)-4*XSIZE/6,MENUYSIZE,
                             "...");
            m_pFeedback->labelsize(8);
            m_pFeedback->callback(&CallbackBase::cb,&box->m_redrawer);
//            feedbackNotePage(); // segfaults on old FLTK (1.0.11), but not 1.1.x



           // NEXT
            but 
                = new Fl_Button(
                    XSIZE-XSIZE/(6*2) ,YSIZE-MENUYSIZE, 
                    XSIZE/(6*2),MENUYSIZE,
                    "@->");
            but->labeltype(FL_SYMBOL_LABEL);
            but->callback( &ApplicationCallback::cb, &m_toNext );
            m_pRightButton = but;

            window->end();

#endif
// END plain FTLK
// ===============================================
            Fl::visual(FL_DOUBLE|FL_INDEX);

            m_pUndoButton->deactivate();
            m_pRedoButton->deactivate();

            window->show();
            return window;
  
        }
//};



void Application::updateUndoStatus(bool status )
{
    //m_pUndoButton->label( (status? "@|<" : "" ) );
    if(status) 
        m_pUndoButton->activate();
    else 
        m_pUndoButton->deactivate();
    //m_pUndoButton->redraw();
}
void Application::updateRedoStatus(bool status )
{
   // m_pRedoButton->label( (1? "@>|" : "" ) );
    if(status) 
        m_pRedoButton->activate();
    else 
        m_pRedoButton->deactivate();

    //m_pRedoButton->redraw();
}










//============== ApplicationWindow

ApplicationWindow::ApplicationWindow( 
    int width, int height, const char* name, Application *pApp )
    :
#ifdef FLPDA 
    Fl_App_Window( width, height, name ), 
#else
 Fl_Double_Window( width, height, name ), 
#endif
    m_pApp( pApp )
{
}

int ApplicationWindow::handleKeyboard(int evt)
{
    int key = Fl::event_key();
    
    switch (key){ 
        
    case FL_Left : 
        if( Fl::event_state(FL_SHIFT) )
            m_pApp->m_noteBackward.callback();
        else
            m_pApp->prevNote(0);

        return 1;
    case FL_Right : 
      if( Fl::event_state(FL_SHIFT) )
            m_pApp->m_noteForward.callback();
        else
            m_pApp->nextNote(0);
        return 1;


    case FL_Page_Up :
        if( Fl::event_state(FL_SHIFT) )
        {
            m_pApp->m_pArea->m_undoer.callback();
        }
        else
        {
            m_pApp->zoom(-1);
            //m_pApp->toolSelect(-1);

        }
        return 1;
    case FL_Page_Down :
        if( Fl::event_state(FL_SHIFT) )
        {
            m_pApp->m_pArea->m_redoer.callback();
        }
        else
        {        
            //m_pApp->zoom(1);
            m_pApp->toolSelect(1);
        }
        return 1;
                
    }

    return Fl_Window::handle(evt);
}


int ApplicationWindow::handle( int event )
{
    
    if( event == FL_KEYBOARD )
    {
        return handleKeyboard(event);
    }
    return Fl_Window::handle(event);
}




// ==================

void setupButton( Fl_Button *but )
{
    but->labelsize(8);
    but->size( XSIZE/(4), Widget_Factory::buttonheight() );   
}

void setupSlider( Fl_Slider *slider )
{
    slider->labelsize(8);
    slider->range( 2.0, 32.0 );
    slider->type( FL_HOR_SLIDER );
    slider->step( 1, 1 );
}

void ConfigController::createWindow()
{
    
    Fl_App_Window* window = Widget_Factory::new_window(_("Config"));
    window->set_modal();


    // Check buttons (use Fl_Light_Button as it is more visible)
    int buttonheight = Widget_Factory::buttonheight();

    m_pHorizCheckButton = new Fl_Light_Button( 
        0,0, XSIZE/2,  buttonheight, _("Horizontal lines")
        );
    m_pHorizCheckButton->labelsize(8);
    m_pHorizCheckButton->selection_color( FL_BLUE );

    m_pVertCheckButton = new Fl_Light_Button( 
        0,buttonheight*2, 
        XSIZE/2,  buttonheight, _("Vertical lines") 
        );
    m_pVertCheckButton->labelsize(8);
    m_pVertCheckButton->selection_color( FL_BLUE );


    // Sliders
    m_pHorizSpace = new Fl_Value_Slider(0,buttonheight,XSIZE,buttonheight);
    setupSlider( m_pHorizSpace );

    m_pVertSpace = new Fl_Value_Slider(0,buttonheight*3,XSIZE,buttonheight);
    setupSlider( m_pVertSpace );


    m_pPointRange = new Fl_Value_Slider(
        0,buttonheight*5,XSIZE,buttonheight,_("Point range"));
    setupSlider( m_pPointRange );
    m_pPointRange->range( 0.0, 16.0 );


    m_pEraseRange = new Fl_Value_Slider(
        0,buttonheight*7,XSIZE,buttonheight,_("Erase range"));
    setupSlider( m_pEraseRange );
    m_pEraseRange->range( 0.0, 16.0 );


    



    // Toolbar

    Fl_Dockable_Window* toolbar = Widget_Factory::new_toolbar();

    Fl_Button *but;

    but = Widget_Factory::new_button(_("Done") );
    setupButton( but );
    but->callback( &CallbackBase::cb, &m_ok);

    but = Widget_Factory::new_button(_("Cancel") );
    setupButton( but );
    but->callback( &CallbackBase::cb, &m_cancel);


    but = Widget_Factory::new_button(_("Reset") );
    setupButton( but );
    but->callback( &CallbackBase::cb, &m_reset);

    toolbar->end();
    
    window->add_dockable(toolbar, 1);
    
    updateView();
    window->show();
    m_pConfigWindow = window;
    //return(window);    
}

void  ConfigController::updateView()
{
    m_pHorizCheckButton->value( m_currentConfig.horizlines );
    m_pVertCheckButton->value(  m_currentConfig.vertlines );
    m_pHorizSpace->value( m_currentConfig.horizlinespace );
    m_pVertSpace->value( m_currentConfig.vertlinespace );
    m_pPointRange->value( m_currentConfig.pointrange );
    m_pEraseRange->value( m_currentConfig.m_eraseRange );

}
void  ConfigController::updateConfig()
{
    m_currentConfig.horizlines = m_pHorizCheckButton->value();
    m_currentConfig.vertlines  = m_pVertCheckButton->value();
    m_currentConfig.horizlinespace = static_cast<int>(m_pHorizSpace->value());
    m_currentConfig.vertlinespace  = static_cast<int>(m_pVertSpace->value());
    m_currentConfig.pointrange     = static_cast<int>(m_pPointRange->value());
    m_currentConfig.m_eraseRange   = static_cast<int>(m_pEraseRange->value());
}


void ConfigController::openWindow()
{
    createWindow();
}


ConfigController::ConfigController( Config &config )
    : m_ok(     this, &ConfigController::ok ), 
      m_cancel( this, &ConfigController::cancel ), 
      m_reset( this, &ConfigController::reset ), 
      
      m_configInUse( config ),
      m_oldConfig( config ), m_currentConfig ( config ),
      m_pConfigWindow( NULL )
{}

int ConfigController::ok(void*)
{
    updateConfig();
    assign( m_currentConfig, m_configInUse );
    // should we force a redraw on the parent?
    cancel(NULL);
}
int ConfigController::cancel(void*)
{
    if(m_pConfigWindow){ 
        delete m_pConfigWindow; 
        m_pConfigWindow = NULL;
    }
}
int ConfigController::reset(void*)
{
    assign( m_oldConfig, m_currentConfig );
    updateView();
}
void ConfigController::assign( Config &arg, Config &rec )
{
    rec.horizlines = arg.horizlines;
    rec.horizlinespace = arg.horizlinespace;
    rec.vertlines = arg.vertlines;
    rec.vertlinespace = arg.vertlinespace;
    rec.pointrange = arg.pointrange;
    rec.m_eraseRange = arg.m_eraseRange;
    rec.m_maxRedoStrokes = arg.m_maxRedoStrokes;
    rec.m_experimental = arg.m_experimental;
}




}; // namespace Prescribble



// Setup the I18N stuff
void vrlang_init( void )
{
	const char* h_dir = getenv("HOME");
	char str[1024];
	sprintf(str, "%s%s", h_dir, "/.fldm");
	char *key, *value, *p;
	
	FILE *preff=fopen( str, "r" );
	if( preff!=0 )
	while( fgets(str,sizeof(str),preff)!=NULL ){
		for(p=str;(*p==' ')||(*p=='\t');p++);
		if( *p == '#' )
			continue;
		key=p;
		for(;(*p>' ')&&(*p!='#');p++);
		if( (*p!=0)&&(*p!='#') ){
			*p=0;
			p++;
		}
		for(;(*p==' ')||(*p=='\t');p++);
		value=p;
		for(;(*p>' ')&&(*p!='#');p++);
		*p=0;
		if( strcmp(key,"lang")==0 ){
			char *envstr;
			sprintf( str, "LANG=%s", value );
			putenv( strdup( str ) );
			break;
		}
	}
	if( preff!=0 )
	fclose( preff );
}


//================= MAIN =======================

int main( int argc, char **argv )
{
    std::printf("Prescribble started. "  VERSION  "\n");

    // I18N
    vrlang_init();    
    setlocale(LC_ALL, "");
    bindtextdomain(PACKAGE, LOCALEDIR);
    textdomain(PACKAGE);

    // Force some words into I18N
    _("Done");


    Prescribble::Config conf;


    // command line arguments
    if(argc>1)
        for(int i = 1;i<argc;i++)
        {
            if( std::strcmp( argv[i],"-hl")==0 ) 
                 conf.horizlines = true;
            if( std::strcmp( argv[i],"-nohl")==0 ) 
                 conf.horizlines = false;
            if( std::strcmp( argv[i],"-hs")==0 && i+1<argc) 
                conf.horizlinespace = std::atoi(argv[i+1]);

            if( std::strcmp( argv[i],"-vl")==0 ) 
                 conf.vertlines = true;
            if( std::strcmp( argv[i],"-novl")==0 ) 
                 conf.vertlines = false;
            if( std::strcmp( argv[i],"-vs")==0 && i+1<argc) 
                conf.vertlinespace = std::atoi(argv[i+1]);

            if( std::strcmp( argv[i],"-auto")==0 )
            { 
                 conf.quietLoadOnEntry = true;
                 conf.quietSaveOnExit  = true; 
            }
            if( std::strcmp( argv[i],"-noauto")==0 )
            { 
                 conf.quietLoadOnEntry = false;
                 conf.quietSaveOnExit  = false; 
            }


            if( std::strcmp( argv[i],"-r")==0 && i+1<argc) 
                conf.pointrange = std::atoi(argv[i+1]);

            if( std::strcmp( argv[i],"-maxredo")==0 && i+1<argc) 
                conf.m_maxRedoStrokes = std::atoi(argv[i+1]);

            if( std::strcmp( argv[i],"-eraserange")==0 && i+1<argc) 
                conf.m_eraseRange = std::atoi(argv[i+1]);
            
            if( std::strcmp( argv[i],"-f")==0 && i+1<argc) 
                conf.filename = argv[i+1];

           if( std::strcmp( argv[i],"-redrawless")==0 ) 
                conf.onlyCheatRedrawDuringMouseDown = false;

           if( std::strcmp( argv[i],"-ziphack")==0 ) 
               conf.allowZipUsage = true;
           if( std::strcmp( argv[i],"-noziphack")==0 ) 
               conf.allowZipUsage = false;

           if( std::strcmp( argv[i],"-X")==0 ) 
               conf.m_experimental = true;

        }

    conf.sanitize();

    Prescribble::Application app(conf);
    app.createArea();   
    app.start();

    return Fl::run();
}



