// #################################
//
//           Prescribble
//
//        (C)opyright 2001 
//           by Kent Dahl 
//
//        MenThal@bigfoot.com
//  http://www.bigfoot.com/~MenThal/
//
//    Released under the GPL 
//     (Gnu Public License)
//
//       Use at own risk. 
//
// ##################################

#ifndef prescrib_fltk_hpp
#define prescrib_fltk_hpp



#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Slider.H>

#include <string>
#include <functional>

#include "my_iostream.hpp"

#include "prescribblearea.hpp"



namespace Prescribble {


class Config;
class PrescribbleArea;
class Application;
class ApplicationWindow;



  

//================
// Callback template
    template<typename T> 
    struct GeneralCallback : public CallbackBase {
        T *m_pThat;
        std::mem_fun1_t<int,T,void*>  m_fptr;
        void *m_data;
        
        GeneralCallback( T *a_pThat,
                         int (T::*a_pFunc)(void*),
                         void *a_data = 0);
        
        void callback();
    };
    
    
    template <typename T> 
    GeneralCallback<T>::GeneralCallback( T *a_pThat,
                                      int (T::*a_pFunc)(void*),
                                      void *a_data )
        : m_pThat( a_pThat ),
          m_fptr( a_pFunc ),
          m_data( a_data )
    {}
    
    template<typename T> 
    void GeneralCallback<T>::callback(){
        m_fptr( m_pThat, m_data );
    }

//================

    typedef GeneralCallback<Application> ApplicationCallback;
    typedef ApplicationCallback PresApp_Callback;



    class ConfigController;
    typedef GeneralCallback<ConfigController> ConfigCallback;
    

   // ConfigController
    
    class ConfigController {
        ConfigCallback m_ok, m_cancel, m_reset;
        Config &m_configInUse;
        Config m_oldConfig, m_currentConfig;
        Fl_Window *m_pConfigWindow;
        Fl_Button *m_pHorizCheckButton, *m_pVertCheckButton;
        Fl_Slider *m_pHorizSpace, *m_pVertSpace, 
            *m_pPointRange, *m_pEraseRange;
    private:
        void createWindow();
        void updateView();
        void updateConfig();
        static void assign( Config &argument, Config &reciever );

    public:
        void openWindow();
        ConfigController( Config &config );
        int ok(void*);
        int reset(void*);
        int cancel(void*);
    };





//================

#ifdef FLPDA
#include <Flek/Fl_App_Window.H>
#endif

class ApplicationWindow :

#ifdef FLPDA
 public Fl_App_Window
#else
 public Fl_Double_Window 
#endif
  {
protected:
    Application *m_pApp;
public:
    ApplicationWindow( int width, int height, 
                   const char* name, Application *pApp );

    int handle(int evt);
    int handleKeyboard(int evt);

};
//================


class Application : public NoteListener {

    typedef std::vector<Note*> NotesVector ;

    NotesVector m_notes;
    int m_currentNote;
    // ConfigController m_configController;

private:
    static void done_cb(Fl_Widget*, void*);
    static Fl_Window* createAboutDialog( const char* application, 
                                         const char* version );

public:
    PrescribbleArea *m_pArea;
    Fl_Window *m_pMainWindow;
    Fl_Button *m_pFeedback;
    char *m_pFeedbackString;

    Fl_Menu_Item *m_pToolboxMenu;
    Fl_Menu_Button *m_pEditMenuButton;



    Fl_Button *m_pUndoButton, *m_pRedoButton;
    Fl_Button *m_pLeftButton, *m_pRightButton;


    Config m_config;

public:
    ApplicationCallback m_savingQuitter, m_nonsavingQuitter, 
        m_loader, m_loaderFrom, m_makeNew,
        m_saver, m_saverAs, m_about, 
        m_toPrev, m_toNext, m_inserter, m_appender, 
        m_noteForward, m_noteBackward,
        m_noteDelete, m_configOpener;


public:
    
    Application( Config &a_conf);
    virtual ~Application(){}
    void start();

    void resetDefaultNotes();
    void removeNotes();


    void writeto( my_ostream & out );

    int handleKeyboard(int evt);

    int saveNotes(void*);
    int saveNotesTo(void*);
    int loadNotesFrom(void*);
    int loadNotes(void*);

    int clearNotes(void*);

    // int quit(void*);
    int saveAndQuit(void*);
    int quitWithoutSaving(void*);
    int about(void*);

    void setFeedbackLabel( const char* );
    void feedbackNotePage();

    int nextNote(void*);
    int prevNote(void*);

    int insertNote(void*);
    int deleteNote(void*);
    int moveNote(void*);

    //int setConfig(void*);
    int openConfig(void*);
    
    void zoom(int);
    void toolSelect(int);

    Fl_Menu_Item* getFilePopupMenu();
    Fl_Menu_Item* getEditPopupMenu(PrescribbleArea *box);
    Fl_Window *createArea( );

   void updateUndoStatus(bool);
    void updateRedoStatus(bool);


};
    typedef Application PrescribbleApp;



}; // namespace Prescribble


#endif
