require 'prescribble'

require 'test/unit'

class TestPoint2D < Test::Unit::TestCase
  include Prescribble
  def set_up
    @p1 = Point2D.new(25,5)
    @p2 = Point2D.new(15,3)
  end

  def test_setup
    assert_equal @p1.x, 25
    assert_equal @p1.y, 5
    assert_equal @p2.x, 15
    assert_equal @p2.y, 3
  end

  def test_manhattan
    assert_equal @p1.manhattan(@p2), 12
  end

  def test_range
    assert @p1.isWithinRange( @p2, 100 )
    assert @p1.isWithinRange( @p2, 10 )
    assert ! @p1.isWithinRange( @p2, 5 )
    assert ! @p1.isWithinRange( @p2, 1 )    
  end

end

class TestRect2D < Test::Unit::TestCase
  include Prescribble

  def set_up
    @p = [0,5,10,15,20,25,30,35,40,45,50].collect{|i| Point2D.new(i,i)}
    @ul = @p[1]  # Point2D.new(5,5)
    @br = @p[5]  # Point2D.new(25,25)
    @r1 = Rect2D.new( @ul, @br )   
  end

  def test_setup
    assert_equal @ul.x, @r1.ul.x
    assert_equal @ul.y, @r1.ul.y
    assert_equal @br.x, @r1.br.x
    assert_equal @br.y, @r1.br.y
    assert @r1.isValid
    assert_equal 21, @r1.width
    assert_equal 21, @r1.height    
  end

  def test_isWithin
    assert @r1.isWithin( @ul )
    assert @r1.isWithin( @br )
    assert @r1.isWithin(  Point2D.new( 15, 15 ) )
    assert !@r1.isWithin( Point2D.new(  0,  0 ) )
    assert !@r1.isWithin( Point2D.new( 30, 30 ) )    
  end

  def test_fenceIn
    @r1.fenceIn( Point2D.new( 15, 3 ) )
    assert_equal 3, @r1.ul.y
    assert_equal 5, @r1.ul.x
    assert_equal 25, @r1.br.y
    assert_equal 25, @r1.br.x       
  end

  def does_collide( index1, index2, type )
    type = eval "Rect2D::#{type}"
    collision = @r1.collides( Rect2D.new( @p[index1], @p[index2] ) )
    type == collision
  end
  
  def test_collides
    assert does_collide(0,0, :NONE)
    assert does_collide(0,1, :OVERLAP)
    assert does_collide(2,3, :OUTSIDE) 
    assert does_collide(0,9, :INSIDE)     
  end

end

# Does not work...
class MockDrawingContext < Prescribble::StrokeDrawingContext
  def drawLine( p1, p2 )
    puts "DRAW LINE"
  end
  def turnWhite
    puts "TURN WHITE"
  end
  def turnBlack
    puts "TURN BLACK"
  end
end


class Point2D
  def to_str
    "(#{x}, #{y})"
  end
end

class TestStroke < Test::Unit::TestCase
  include Prescribble

  def set_up
    @s1 = Stroke.new
    (1..9).each{|i| @s1.add( Point2D.new(i*10,i*10) ) }
  end

  def test_getBoundingRect
    assert_equal 9, @s1.m_points.length
    b = @s1.getBoundingRect
    assert_equal 10, b.ul.x
    assert_equal 10, b.ul.y
    assert_equal 90, b.br.x
    assert_equal 90, b.br.y    
  end



  def print_strokes( strokes )
    strokes = reduce_strokes(strokes) unless strokes.kind_of? Array
    print "<",strokes.collect{|stroke| 
      "["+stroke.collect{|p| "(#{p.x}, #{p.y})" }.join('-')+"]"
    }.join(", "),">\n"
  end

  def reduce_strokes( strokes )
    result = []
    strokes.each_stroke{|s|
      new_stroke = []
      s.m_points.each{|p| new_stroke << p }
      result << new_stroke
    }
    result
  end

  def test_clippingAway
    clip = Rect2D.new( @s1.m_points[2], @s1.m_points[6] )
    strokes = @s1.clippingAway( clip )
 
    print_strokes strokes 

    strokes = @s1.clippingAway( clip, true )
    print_strokes strokes

  end

end


