#include "prescribble.hpp"
#include "presaction.hpp"

#include <vector>

/**
 * Using CppUnit, the C++ unit testing framework; 
 * see: https://sourceforge.net/projects/cppunit
 */

#include <cppunit/TestCase.h>
#include <cppunit/TestCaller.h>
#include <cppunit/TextTestResult.h>
#include <cppunit/TestSuite.h>
#include <cppunit/TestRegistry.h>


using namespace CppUnit;

class PrescribbleTest : public TestCase {

    PresNoteConfig m_config;
    PrescribbleNote *m_pNote;

public:
    PrescribbleTest( ) 
        : TestCase(), m_config( 16, 10 ) 
        {}

    void setUp()
        {
            m_pNote = new PrescribbleNote( m_config );
        }
    void tearDown()
        {
            delete m_pNote;
        }

    void testSimpleUndoRedo()
        {
            m_pNote->beginStroke(    Point2D(5,5)     );
            m_pNote->continueStroke( Point2D(100,5)   );
            m_pNote->continueStroke( Point2D(100,100) );
            m_pNote->endStroke(      Point2D(5,100)   );
            
            assertImplementation( m_pNote->anyUndosLeft(), "added ok",
                                  __LINE__, __FILE__ );
            assertImplementation( !m_pNote->anyRedosLeft(), "added ok",
                                  __LINE__, __FILE__ );


            m_pNote->undoAction();
            assertImplementation( !m_pNote->anyUndosLeft(), "undone ok",
                                  __LINE__, __FILE__ );
            assertImplementation( m_pNote->anyRedosLeft(), "undone ok",
                                  __LINE__, __FILE__ );

            m_pNote->redoAction();
            assertImplementation( m_pNote->anyUndosLeft(), "redone ok",
                                  __LINE__, __FILE__ );
            assertImplementation( !m_pNote->anyRedosLeft(), "redone ok",
                                  __LINE__, __FILE__ );



        }


    void testSimpleBoxCut()
        {

            assertImplementation( !m_pNote->anyUndosLeft(), "prestate",
                                  __LINE__, __FILE__ );

            m_pNote->beginStroke(    Point2D(5,5)     );
            m_pNote->continueStroke( Point2D(55,5)     );
            m_pNote->continueStroke( Point2D(100,5)   );
            m_pNote->continueStroke( Point2D(100,100) );
            m_pNote->continueStroke( Point2D(55,100)     );
            m_pNote->endStroke(      Point2D(5,100)   );


            std::cout<<"Pre erase action \n";
            PresAction *action = 
                m_pNote->eraseRectAction( Rect2D( Point2D( 50,5),
                                                  Point2D( 60,100 ) ) );
            
            PresComplexAction *comact 
                = dynamic_cast<PresComplexAction*>(action);
            
            assertImplementation( comact, "made complex", 
                                  __LINE__, __FILE__ );
            assertImplementation( !comact->empty(), "has sub actions", 
                                  __LINE__, __FILE__ );
            assertEquals( 4, comact->size(),
                          __LINE__, __FILE__ );


            assertImplementation( !m_pNote->anyRedosLeft(), "prestate",
                                  __LINE__, __FILE__ );

            m_pNote->doAction( *action );

            assertImplementation( m_pNote->anyUndosLeft(), "cut ok",
                                  __LINE__, __FILE__ );
            assertImplementation( !m_pNote->anyRedosLeft(), "cut ok",
                                  __LINE__, __FILE__ );

            m_pNote->undoAction();

            assertImplementation( m_pNote->anyUndosLeft(), "undo cut",
                                  __LINE__, __FILE__ );
            assertImplementation( m_pNote->anyRedosLeft(), "undo cut",
                                  __LINE__, __FILE__ );

            m_pNote->undoAction();

            assertImplementation( !m_pNote->anyUndosLeft(), "undo draw",
                                  __LINE__, __FILE__ );
            assertImplementation( m_pNote->anyRedosLeft(), "undo draw",
                                  __LINE__, __FILE__ );



            
            


            
            


        }

                



    virtual void registerTests(TestSuite *suite)
        {
            suite->addTest(
                new TestCaller<PrescribbleTest>(
                    "simple undo redo",
                    &PrescribbleTest::testSimpleUndoRedo ));
            suite->addTest(
                new TestCaller<PrescribbleTest>(
                    "box cut",
                    &PrescribbleTest::testSimpleBoxCut ));

        }

};


PrescribbleTest test;


int main()
{

    TextTestResult res;
    TestSuite suite;

    test.registerTests(&suite);
    
    std::vector<CppUnit::Test*> tests = 
        CppUnit::TestRegistry::getRegistry().getAllTests();
    
    for(std::vector<CppUnit::Test*>::iterator it=tests.begin();
        it!=tests.end(); ++it) {
        
        (*it)->run(&res);
        
    }
    std::cout << res << endl;

 
}
